/**
 * Represents a stock quote.
 */
export class Quote {
    readonly symbol: string;
    readonly askPrice: number;
    readonly askSize: number;
    readonly bidPrice: number;
    readonly bidSize: number;
    readonly timeStamp: Date;

    constructor(symbol: string, askPrice: number, askSize: number, bidPrice: number, bidSize: number, timeStamp: Date) {
        this.symbol = symbol;
        this.askPrice = askPrice;
        this.askSize = askSize;
        this.bidPrice = bidPrice;
        this.bidSize = bidSize;
        this.timeStamp = timeStamp;
    }
}

/**
 * A provider for fetching stock quotes.
 */
export interface QuoteProvider {

    /**
     * Fetches a quote for the given stock symbol.
     * @param symbol The stock symbol to fetch the quote for.
     * @returns A Promise that resolves to a Quote object.
     */
    readonly fetchQuote: (symbol: string) => Promise<Quote>;
}
